@@ -186,7 +186,7 @@ tools = [
 ['grid_tools', 19, 'Grid Orientation'],
 ['grid_tools', 20, 'Combine Grids'],
 ['grid_tools', 21, 'Grid Cell Index'],
-['grid_tools', 22, 'Grids from classified grid and table'],
+['grid_tools', 22, 'Grids from Classified Grid and Table'],
 ['grid_tools', 23, 'Create Grid System'],
 ['grid_tools', 24, 'Grid Masking'],
 ['grid_tools', 25, 'Close Gaps with Spline'],
@@ -532,7 +532,7 @@ tools = [
 ['ta_lighting', 3, 'Sky View Factor'],
 ['ta_lighting', 4, 'Topographic Correction'],
 ['ta_lighting', 5, 'Topographic Openness'],
-['ta_lighting', 6, 'Visibility (points)'],
+['ta_lighting', 6, 'Visibility Analysis'],
 ['ta_morphometry', 0, 'Slope, Aspect, Curvature'],
 ['ta_morphometry', 1, 'Convergence Index'],
 ['ta_morphometry', 2, 'Convergence Index (Search Radius)'],
@@ -815,5 +815,6 @@ tools = [
 ['io_pdal', 1, 'Export LAS/LAZ File'],
 ['shapes_lines', 11, 'Split Lines'],
 ['sim_hydrology', 12, 'Glugla Coefficient'],
-['statistics_grid', 21, 'Grid Histogram']
+['statistics_grid', 21, 'Grid Histogram'],
+['imagery_tools', 16, 'Import Sentinel-3 OLCI Scene']
 ]
