@@ -77,8 +77,13 @@ long		SG_Color_Get_Random	(void)
 
 //---------------------------------------------------------
 bool		SG_Color_From_Text	(const CSG_String &Text, long &Color)
-{	// from wx/colourmng.cpp, hexadecimal prefixed with # ("HTML syntax") see https://drafts.csswg.org/css-color/#hex-notation
+{	// from wx/src/common/colourcmn.cpp, hexadecimal prefixed with # ("HTML syntax") see https://drafts.csswg.org/css-color/#hex-notation
 
+    if( Text.is_Empty() || Text.Get_Char(0) != L'#' )
+    {
+        return( false );
+    }
+    
 	const char	*s	= Text.b_str();	unsigned long	c;
 
 	if( sscanf(s + 1, "%lx", &c) == 1 )
@@ -86,8 +91,13 @@ bool		SG_Color_From_Text	(const CSG_String &Text, long &Color)
 		switch( Text.Length() - 1 )
 		{
 		case 6: // #rrggbb
-			Color	= (long)((c << 8) + 0xFF);
-			return( true );
+			c	    = (c << 8) + 0xFF;
+            Color   = SG_GET_RGB(
+                (unsigned char)((c >> 24) & 0xFF),
+                (unsigned char)((c >> 16) & 0xFF),
+                (unsigned char)((c >>  8) & 0xFF)
+            );
+            return( true );
 
 		case 8: // #rrggbbaa
 			Color	= SG_GET_RGBA(
@@ -99,8 +109,13 @@ bool		SG_Color_From_Text	(const CSG_String &Text, long &Color)
 			return( true );
 
 		case 3: // #rgb
-			Color	= (long)((c << 4) + 0xF);
-			return( true );
+			c	    = (c << 4) + 0xF;
+            Color	= SG_GET_RGB(
+                (unsigned char)(((c >> 12) & 0xF) * 0x11),
+                (unsigned char)(((c >>  8) & 0xF) * 0x11),
+                (unsigned char)(((c >>  4) & 0xF) * 0x11)
+            );
+            return( true );
 
 		case 4: // #rgba
 			Color	= SG_GET_RGBA(
@@ -110,6 +125,9 @@ bool		SG_Color_From_Text	(const CSG_String &Text, long &Color)
 				(unsigned char)(((c      ) & 0xF) * 0x11)
 			);
 			return( true );
+
+        default:
+            return( false ); // unrecognized
 		}
 	}
 
@@ -430,6 +448,7 @@ const SG_Char * CSG_Colors::Get_Predefined_Name(int Index)
 	case SG_COLORS_YELLOW_RED     :	return( _TL("yellow > red"             ) );
 	case SG_COLORS_YELLOW_GREEN   :	return( _TL("yellow > green"           ) );
 	case SG_COLORS_YELLOW_BLUE    :	return( _TL("yellow > blue"            ) );
+	case SG_COLORS_GREEN_RED      :	return( _TL("green > red"              ) );
 	case SG_COLORS_RED_GREEN      :	return( _TL("red > green"              ) );
 	case SG_COLORS_RED_BLUE       :	return( _TL("red > blue"               ) );
 	case SG_COLORS_GREEN_BLUE     :	return( _TL("green > blue"             ) );
@@ -521,13 +540,20 @@ bool CSG_Colors::Set_Predefined(int Index, bool bRevert, int nColors)
 		Set_Color(2, SG_GET_RGB(  0,  64, 127));
 		break;
 
-	case SG_COLORS_RED_GREEN:
+	case SG_COLORS_GREEN_RED:
 		Set_Count(3);
 		Set_Color(0, SG_GET_RGB(  0, 127,   0));
 		Set_Color(1, SG_GET_RGB(255, 255,   0));
 		Set_Color(2, SG_GET_RGB(255,   0,   0));
 		break;
 
+	case SG_COLORS_RED_GREEN:
+		Set_Count(3);
+		Set_Color(0, SG_GET_RGB(200,   0,   0));
+		Set_Color(1, SG_GET_RGB(255, 255, 127));
+		Set_Color(2, SG_GET_RGB(  0,  63,   0));
+		break;
+
 	case SG_COLORS_RED_BLUE:
 		Set_Ramp(SG_GET_RGB(255,   0,   0), SG_GET_RGB(  0,   0, 255));
 		break;
