@@ -1245,6 +1245,7 @@ enum ESG_Colors
 	SG_COLORS_YELLOW_RED,
 	SG_COLORS_YELLOW_GREEN,
 	SG_COLORS_YELLOW_BLUE,
+	SG_COLORS_GREEN_RED,
 	SG_COLORS_RED_GREEN,
 	SG_COLORS_RED_BLUE,
 	SG_COLORS_GREEN_BLUE,
@@ -1469,6 +1470,7 @@ typedef enum ESG_UI_Callback_ID
 {
 	CALLBACK_PROCESS_GET_OKAY,
 	CALLBACK_PROCESS_SET_OKAY,
+	CALLBACK_PROCESS_SET_BUSY,
 	CALLBACK_PROCESS_SET_PROGRESS,
 	CALLBACK_PROCESS_SET_READY,
 	CALLBACK_PROCESS_SET_TEXT,
@@ -1536,6 +1538,7 @@ SAGA_API_DLL_EXPORT int						SG_UI_Progress_Lock			(bool bOn);
 SAGA_API_DLL_EXPORT int						SG_UI_Progress_Reset		(void);
 SAGA_API_DLL_EXPORT bool					SG_UI_Process_Get_Okay		(bool bBlink = false);
 SAGA_API_DLL_EXPORT bool					SG_UI_Process_Set_Okay		(bool bOkay = true);
+SAGA_API_DLL_EXPORT bool					SG_UI_Process_Set_Busy		(bool bOn = true, const CSG_String &Message = "");
 SAGA_API_DLL_EXPORT bool					SG_UI_Process_Set_Progress	(double Position, double Range);
 SAGA_API_DLL_EXPORT bool					SG_UI_Process_Set_Ready		(void);
 SAGA_API_DLL_EXPORT void					SG_UI_Process_Set_Text		(const CSG_String &Text);
