@@ -49,6 +49,10 @@
 //---------------------------------------------------------
 #include "3d_view_tools.h"
 
+#include "sgdi_helper.h"
+
+#include <wx/dcmemory.h>
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -76,14 +80,16 @@ enum
 //---------------------------------------------------------
 CSG_3DView_Canvas::CSG_3DView_Canvas(void)
 {
-	m_pDrape		= NULL;
-	m_Image_pRGB	= NULL;
+	m_pDrape     = NULL;
+	m_Image_pRGB = NULL;
 
-	m_bgColor		= SG_COLOR_WHITE;
-	m_bBox			= true;
-	m_BoxBuffer		= 0.01;
-	m_bStereo		= false;
-	m_dStereo		= 2.0;
+	m_bgColor    = SG_COLOR_WHITE;
+	m_bBox       = true;
+	m_bLabels    = false;
+	m_bNorth     = false;
+	m_BoxBuffer  = 0.01;
+	m_bStereo    = false;
+	m_dStereo    = 2.;
 }
 
 
@@ -94,10 +100,10 @@ CSG_3DView_Canvas::CSG_3DView_Canvas(void)
 //---------------------------------------------------------
 void CSG_3DView_Canvas::Set_Image(BYTE *pRGB, int NX, int NY)
 {
-	m_Image_pRGB	= pRGB;
+	m_Image_pRGB = pRGB;
 
-	m_Image_NX		= NX;
-	m_Image_NY		= NY;
+	m_Image_NX   = NX;
+	m_Image_NY   = NY;
 
 	m_Image_zMax.Create(m_Image_NX, m_Image_NY);
 
@@ -153,13 +159,14 @@ bool CSG_3DView_Canvas::Draw(void)
 	//-------------------------------------------------
 	if( m_bStereo == false )
 	{
-		m_Image_zMax.Assign(999999.0);
+		m_Image_zMax.Assign(999999.);
 
 		m_Color_Mode	= COLOR_MODE_RGB;
 
 		On_Draw();
 
 		_Draw_Box();
+		_Draw_Labels();
 	}
 
 	//-----------------------------------------------------
@@ -171,7 +178,7 @@ bool CSG_3DView_Canvas::Draw(void)
 		double	xShift	= m_Projector.Get_xShift();
 
 		//-------------------------------------------------
-		m_Image_zMax.Assign(999999.0);
+		m_Image_zMax.Assign(999999.);
 
 		m_Projector.Set_xShift   (xShift  - dShift );
 		m_Projector.Set_yRotation(yRotate - dRotate);
@@ -181,9 +188,10 @@ bool CSG_3DView_Canvas::Draw(void)
 		On_Draw();
 
 		_Draw_Box();
+		_Draw_Labels();
 
 		//-------------------------------------------------
-		m_Image_zMax.Assign(999999.0);
+		m_Image_zMax.Assign(999999.);
 
 		m_Projector.Set_xShift   (xShift  + dShift );
 		m_Projector.Set_yRotation(yRotate + dRotate);
@@ -193,6 +201,7 @@ bool CSG_3DView_Canvas::Draw(void)
 		On_Draw();
 
 		_Draw_Box();
+		_Draw_Labels();
 
 		//-------------------------------------------------
 		m_Projector.Set_xShift   (xShift );
@@ -216,7 +225,7 @@ void CSG_3DView_Canvas::_Draw_Background(void)
 
 	if( m_bStereo )	// greyscale
 	{
-		r	= g	= b	= (int)((SG_GET_R(m_bgColor) + SG_GET_G(m_bgColor) + SG_GET_B(m_bgColor)) / 3.0);
+		r	= g	= b	= (int)((SG_GET_R(m_bgColor) + SG_GET_G(m_bgColor) + SG_GET_B(m_bgColor)) / 3.);
 	}
 	else
 	{
@@ -244,6 +253,31 @@ void CSG_3DView_Canvas::_Draw_Background(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+void CSG_3DView_Canvas::_Draw_Get_Box(TSG_Point_Z Box[8], bool bProjected)
+{
+	TSG_Point_Z	Buffer;
+
+	Buffer.x = m_BoxBuffer * (m_Data_Max.x - m_Data_Min.x);
+	Buffer.y = m_BoxBuffer * (m_Data_Max.y - m_Data_Min.y);
+	Buffer.z = m_BoxBuffer * (m_Data_Max.z - m_Data_Min.z);
+
+	Box[0].x = Box[4].x = m_Data_Min.x - Buffer.x; Box[0].y = Box[4].y = m_Data_Min.y - Buffer.y;
+	Box[1].x = Box[5].x = m_Data_Max.x + Buffer.x; Box[1].y = Box[5].y = m_Data_Min.y - Buffer.y;
+	Box[2].x = Box[6].x = m_Data_Max.x + Buffer.x; Box[2].y = Box[6].y = m_Data_Max.y + Buffer.y;
+	Box[3].x = Box[7].x = m_Data_Min.x - Buffer.x; Box[3].y = Box[7].y = m_Data_Max.y + Buffer.y;
+	Box[0].z = Box[1].z = Box[2].z = Box[3].z = m_Data_Min.z - Buffer.z;
+	Box[4].z = Box[5].z = Box[6].z = Box[7].z = m_Data_Max.z + Buffer.z;
+
+	if( bProjected )
+	{
+		for(int i=0; i<8; i++)
+		{
+			m_Projector.Get_Projection(Box[i]);
+		}
+	}
+}
+
 //---------------------------------------------------------
 void CSG_3DView_Canvas::_Draw_Box(void)
 {
@@ -252,39 +286,140 @@ void CSG_3DView_Canvas::_Draw_Box(void)
 		return;
 	}
 
-	int	color	= SG_GET_RGB(SG_GET_R(m_bgColor) + 128, SG_GET_G(m_bgColor) + 128, SG_GET_B(m_bgColor) + 128);
+	int Color = SG_GET_RGB(SG_GET_R(m_bgColor) + 128, SG_GET_G(m_bgColor) + 128, SG_GET_B(m_bgColor) + 128);
 
-	TSG_Point_Z	p[2][4], Buffer;
+	TSG_Point_Z	Box[8]; _Draw_Get_Box(Box, true);
 
-	Buffer.x	= m_BoxBuffer * (m_Data_Max.x - m_Data_Min.x);
-	Buffer.y	= m_BoxBuffer * (m_Data_Max.y - m_Data_Min.y);
-	Buffer.z	= m_BoxBuffer * (m_Data_Max.z - m_Data_Min.z);
-
-	for(int i=0; i<2; i++)
+	for(int i=0; i<8; i+=4)
 	{
-		p[i][0].x	= m_Data_Min.x - Buffer.x;	p[i][0].y	= m_Data_Min.y - Buffer.y;
-		p[i][1].x	= m_Data_Max.x + Buffer.x;	p[i][1].y	= m_Data_Min.y - Buffer.y;
-		p[i][2].x	= m_Data_Max.x + Buffer.x;	p[i][2].y	= m_Data_Max.y + Buffer.y;
-		p[i][3].x	= m_Data_Min.x - Buffer.x;	p[i][3].y	= m_Data_Max.y + Buffer.y;
-		p[i][0].z	= p[i][1].z = p[i][2].z = p[i][3].z = i == 0
-			? m_Data_Min.z - Buffer.z
-			: m_Data_Max.z + Buffer.z;
-
-		for(int j=0; j<4; j++)
-		{
-			m_Projector.Get_Projection(p[i][j]);
-		}
-
-		Draw_Line(p[i][0], p[i][1], color);
-		Draw_Line(p[i][1], p[i][2], color);
-		Draw_Line(p[i][2], p[i][3], color);
-		Draw_Line(p[i][3], p[i][0], color);
+		Draw_Line(Box[i + 0], Box[i + 1], Color);
+		Draw_Line(Box[i + 1], Box[i + 2], Color);
+		Draw_Line(Box[i + 2], Box[i + 3], Color);
+		Draw_Line(Box[i + 3], Box[i + 0], Color);
 	}
 
-	Draw_Line(p[0][0], p[1][0], color);
-	Draw_Line(p[0][1], p[1][1], color);
-	Draw_Line(p[0][2], p[1][2], color);
-	Draw_Line(p[0][3], p[1][3], color);
+	Draw_Line(Box[0], Box[4], Color);
+	Draw_Line(Box[1], Box[5], Color);
+	Draw_Line(Box[2], Box[6], Color);
+	Draw_Line(Box[3], Box[7], Color);
+}
+
+//---------------------------------------------------------
+void CSG_3DView_Canvas::_Draw_Labels(void)
+{
+	if( !m_bLabels )
+	{
+		return;
+	}
+
+	TSG_Point_Z	Box[8]; _Draw_Get_Box(Box, false);
+
+	double Scale = (Box[1].x - Box[0].x) / 1000.;
+
+	_Draw_Label(Scale, Box[0].x, Box[1].x, Box[0],   0,   0,   0);
+	_Draw_Label(Scale, Box[1].y, Box[2].y, Box[1],  90,   0,   0);
+	_Draw_Label(Scale, Box[2].x, Box[3].x, Box[2], 180,   0,   0);
+	_Draw_Label(Scale, Box[3].y, Box[0].y, Box[3], 270,   0,   0);
+
+//	Scale = (Box[4].z - Box[0].z) / 1000.;
+//	_Draw_Label(Scale, Box[0].z, Box[4].z, Box[0],   0, -90,   0);
+}
+
+//---------------------------------------------------------
+void CSG_3DView_Canvas::_Draw_Label(double Scale, double valMin, double valMax, const TSG_Point_Z &Point, double Rx, double Ry, double Rz)
+{
+	bool bAscending = valMax > valMin; if( !bAscending ) { double val = valMin; valMin = valMax; valMax = val; }
+
+	wxSize Size((valMax - valMin) / Scale, 50);//400 / Scale);
+
+	if( Size.GetWidth() < 1 || Size.GetHeight() < 1 )
+	{
+		return;
+	}
+
+	wxBitmap Bitmap(Size, 32); wxMemoryDC dc(Bitmap);
+
+	wxColour FGColor(SG_GET_R(m_bgColor) + 128, SG_GET_G(m_bgColor) + 128, SG_GET_B(m_bgColor) + 128);
+	wxColour BGColor(SG_GET_R(m_bgColor)      , SG_GET_G(m_bgColor)      , SG_GET_B(m_bgColor)      );
+
+	dc.SetBackground(BGColor); dc.Clear(); dc.SetPen(FGColor); dc.SetTextForeground(FGColor);
+	if( !m_bBox ) { dc.DrawLine(0, 0, Size.GetWidth(), 0); }
+
+	Draw_Scale(dc, Size, valMin, valMax, true, bAscending, true);
+
+	CSG_Vector P(3); P[0] = Point.x; P[1] = Point.y; P[2] = Point.z;
+	CSG_Matrix R = SG_Matrix_Get_Rotation(Rx, Ry, Rz, true); R *= Scale;
+
+	wxImage Image(Bitmap.ConvertToImage()); _Draw_Image(Image, P, R, m_bgColor);
+}
+
+//---------------------------------------------------------
+void CSG_3DView_Canvas::_Draw_Image(wxImage &Image, const CSG_Vector &Move, const CSG_Matrix &Rotate, int BGColor)
+{
+	#define GET_NODE(xImage, yImage, node, color) {\
+		CSG_Vector v(3); v[0] = xImage; v[1] = -yImage; v[2] = 0; v = Move + Rotate * v;\
+		TSG_Point_Z	point; point.x = v[0]; point.y = v[1]; point.z = v[2];\
+		m_Projector.Get_Projection(point);\
+		node.x = point.x; node.y = point.y; node.z = point.z; node.c = color;\
+	}
+
+	//-----------------------------------------------------
+	CSG_Grid *pDrape = m_pDrape; m_pDrape = NULL;
+
+	//-----------------------------------------------------
+	#ifndef _DEBUG
+	#pragma omp parallel for
+	#endif
+	for(int y1=1; y1<Image.GetHeight(); y1++)
+	{
+		int y0 = y1 - 1;
+
+		for(int x0=0, x1=1; x1<Image.GetWidth(); x0++, x1++)
+		{
+			int c[4];
+
+			c[0] = SG_GET_RGB(Image.GetRed(x0, y0), Image.GetGreen(x0, y0), Image.GetBlue(x0, y0));
+			c[1] = SG_GET_RGB(Image.GetRed(x1, y0), Image.GetGreen(x1, y0), Image.GetBlue(x1, y0));
+			c[2] = SG_GET_RGB(Image.GetRed(x1, y1), Image.GetGreen(x1, y1), Image.GetBlue(x1, y1));
+			c[3] = SG_GET_RGB(Image.GetRed(x0, y1), Image.GetGreen(x0, y1), Image.GetBlue(x0, y1));
+
+			int n = (c[0] != BGColor ? 1 : 0)
+				  +	(c[1] != BGColor ? 1 : 0)
+				  + (c[2] != BGColor ? 1 : 0)
+				  + (c[3] != BGColor ? 1 : 0);
+
+			if( n > 0 )
+			{
+				TSG_Triangle_Node p[4], Triangle[3];
+
+				GET_NODE(x0, y0, p[0], c[0]);
+				GET_NODE(x1, y0, p[1], c[1]);
+				GET_NODE(x1, y1, p[2], c[2]);
+				GET_NODE(x0, y1, p[3], c[3]);
+
+				int r = (SG_GET_R(c[0]) + SG_GET_R(c[1]) + SG_GET_R(c[2]) + SG_GET_R(c[3])) / 4;
+				int g = (SG_GET_G(c[0]) + SG_GET_G(c[1]) + SG_GET_G(c[2]) + SG_GET_G(c[3])) / 4;
+				int b = (SG_GET_B(c[0]) + SG_GET_B(c[1]) + SG_GET_B(c[2]) + SG_GET_B(c[3])) / 4;
+
+				Triangle[2].x = (p[0].x + p[1].x + p[2].x + p[3].x) / 4.;
+				Triangle[2].y = (p[0].y + p[1].y + p[2].y + p[3].y) / 4.;
+				Triangle[2].z = (p[0].z + p[1].z + p[2].z + p[3].z) / 4.;
+				Triangle[2].c = SG_GET_RGB(
+					r > 255 ? 255 : r < 0 ? 0 : r,
+					g > 255 ? 255 : g < 0 ? 0 : g,
+					b > 255 ? 255 : b < 0 ? 0 : b
+				);
+
+				Triangle[0] = p[0];
+				Triangle[1] = p[1]; Draw_Triangle(Triangle, true);
+				Triangle[0] = p[2]; Draw_Triangle(Triangle, true);
+				Triangle[1] = p[3]; Draw_Triangle(Triangle, true);
+				Triangle[0] = p[0]; Draw_Triangle(Triangle, true);
+			}
+		}
+	}
+
+	m_pDrape = pDrape;
 }
 
 
@@ -299,43 +434,43 @@ int CSG_3DView_Canvas::Get_Color(double Value)
 }
 
 //---------------------------------------------------------
-int CSG_3DView_Canvas::Dim_Color(int Color, double dim)
+int CSG_3DView_Canvas::Dim_Color(int Color, double Dim)
 {
-	if( dim <= 0.0 )
+	if( Dim > 0. )
 	{
-		return( m_bgColor );
+		if( Dim != 1. )
+		{
+			int r = SG_GET_R(m_bgColor) + (int)(Dim * (SG_GET_R(Color) - SG_GET_R(m_bgColor)));
+			int g = SG_GET_G(m_bgColor) + (int)(Dim * (SG_GET_G(Color) - SG_GET_G(m_bgColor)));
+			int b = SG_GET_B(m_bgColor) + (int)(Dim * (SG_GET_B(Color) - SG_GET_B(m_bgColor)));
+
+			Color = SG_GET_RGB(r > 255 ? 255 : r, g > 255 ? 255 : g, b > 255 ? 255 : b);
+		}
+
+		return( Color );
 	}
 
-	if( dim != 1.0 )
-	{
-		int	r	= SG_GET_R(m_bgColor) + (int)(dim * (SG_GET_R(Color) - SG_GET_R(m_bgColor))); if( r > 255 ) r = 255;
-		int	g	= SG_GET_G(m_bgColor) + (int)(dim * (SG_GET_G(Color) - SG_GET_G(m_bgColor))); if( g > 255 ) g = 255;
-		int	b	= SG_GET_B(m_bgColor) + (int)(dim * (SG_GET_B(Color) - SG_GET_B(m_bgColor))); if( b > 255 ) b = 255;
-
-		Color	= SG_GET_RGB(r, g, b);
-	}
-
-	return( Color );
+	return( m_bgColor );
 }
 
 //---------------------------------------------------------
-int CSG_3DView_Canvas::_Dim_Color(int Color, double dim)
+int CSG_3DView_Canvas::_Dim_Color(int Color, double Dim)
 {
-	if( dim <= 0.0 )
+	if( Dim > 0. )
 	{
-		return( 0 );
+		if( Dim != 1. )
+		{
+			int r = (int)(Dim * SG_GET_R(Color));
+			int g = (int)(Dim * SG_GET_G(Color));
+			int b = (int)(Dim * SG_GET_B(Color));
+
+			Color = SG_GET_RGB(r > 255 ? 255 : r, g > 255 ? 255 : g, b > 255 ? 255 : b);
+		}
+
+		return( Color );
 	}
 
-	if( dim != 1.0 )
-	{
-		int	r	= (int)(dim * SG_GET_R(Color)); if( r > 255 ) r = 255;
-		int	g	= (int)(dim * SG_GET_G(Color)); if( g > 255 ) g = 255;
-		int	b	= (int)(dim * SG_GET_B(Color)); if( b > 255 ) b = 255;
-
-		Color	= SG_GET_RGB(r, g, b);
-	}
-
-	return( Color );
+	return( 0 );
 }
 
 //---------------------------------------------------------
@@ -343,35 +478,35 @@ inline void CSG_3DView_Canvas::_Draw_Pixel(int x, int y, double z, int color)
 {
 	if( x >= 0 && x < m_Image_NX && y >= 0 && y < m_Image_NY && z < m_Image_zMax[y][x] )
 	{
-		BYTE	*RGB	= m_Image_pRGB + 3 * (y * m_Image_NX + x);
+		BYTE *RGB = m_Image_pRGB + 3 * (y * m_Image_NX + x);
 
 		switch( m_Color_Mode )
 		{
 		case COLOR_MODE_RGB:
-			RGB[0]	= SG_GET_R(color);
-			RGB[1]	= SG_GET_G(color);
-			RGB[2]	= SG_GET_B(color);
+			RGB[0] = SG_GET_R(color);
+			RGB[1] = SG_GET_G(color);
+			RGB[2] = SG_GET_B(color);
 			break;
 
 		case COLOR_MODE_RED:
-			RGB[0]	= (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
+			RGB[0] = (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
 			break;
 
 		case COLOR_MODE_GREEN:
-			RGB[1]	= (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
+			RGB[1] = (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
 			break;
 
 		case COLOR_MODE_BLUE:
-			RGB[2]	= (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
+			RGB[2] = (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
 			break;
 
 		case COLOR_MODE_CYAN:
-			RGB[1]	= 
-			RGB[2]	= (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
+			RGB[1] = 
+			RGB[2] = (SG_GET_R(color) + SG_GET_G(color) + SG_GET_B(color)) / 3;
 			break;
 		}
 
-		m_Image_zMax[y][x]	= z;
+		m_Image_zMax[y][x] = z;
 	}
 }
 
@@ -381,25 +516,22 @@ inline void CSG_3DView_Canvas::_Draw_Pixel(int x, int y, double z, int color)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CSG_3DView_Canvas::Draw_Point(int x, int y, double z, int color, int size)
+void CSG_3DView_Canvas::Draw_Point(int x, int y, double z, int Color, int Size)
 {
-	if( z > 0.0 )
+	if( z > 0. )
 	{
-		_Draw_Pixel(x, y, z, color);
+		_Draw_Pixel(x, y, z, Color);
 
-		if( size > 0 && size < 50 )
+		if( Size > 0 && Size < 50 )
 		{
-			for(int iy=1; iy<=size; iy++)
+			for(int iy=1; iy<=Size; iy++) for(int ix=0; ix<=Size; ix++)
 			{
-				for(int ix=0; ix<=size; ix++)
+				if( ix*ix + iy*iy <= Size*Size )
 				{
-					if( ix*ix + iy*iy <= size*size )
-					{
-						_Draw_Pixel(x + ix, y + iy, z, color);
-						_Draw_Pixel(x + iy, y - ix, z, color);
-						_Draw_Pixel(x - ix, y - iy, z, color);
-						_Draw_Pixel(x - iy, y + ix, z, color);
-					}
+					_Draw_Pixel(x + ix, y + iy, z, Color);
+					_Draw_Pixel(x + iy, y - ix, z, Color);
+					_Draw_Pixel(x - ix, y - iy, z, Color);
+					_Draw_Pixel(x - iy, y + ix, z, Color);
 				}
 			}
 		}
@@ -417,33 +549,23 @@ void CSG_3DView_Canvas::Draw_Line(double ax, double ay, double az, double bx, do
 	if(	(ax < 0 && bx < 0) || (ax >= m_Image_NX && bx >= m_Image_NX)
 	||	(ay < 0 && by < 0) || (ay >= m_Image_NY && by >= m_Image_NY) )
 	{
-		return;
+		return; // completely out of area
 	}
 
-	double	dx	= bx - ax;
-	double	dy	= by - ay;
-	double	dz	= bz - az;
-
-	if( bz < 0.0 || az < 0.0 )
+	if( bz < 0. && az < 0. )
 	{
-		return;
+		return; // completely in front of screen
 	}
 
-	double	n;
+	double n, dx = bx - ax, dy = by - ay, dz = bz - az;
 
-	if( fabs(dx) > fabs(dy) && fabs(dx) > 0.0 )
+	if( fabs(dx) > fabs(dy) && fabs(dx) > 0. )
 	{
-		n	 = fabs(dx);
-		dx	 = dx < 0 ? -1 : 1;
-		dy	/= n;
-		dz	/= n;
+		n = fabs(dx); dx = dx < 0. ? -1. : 1.; dy /= n; dz /= n;
 	}
-	else if( fabs(dy) > 0.0 )
+	else if( fabs(dy) > 0. )
 	{
-		n	 = fabs(dy);
-		dx	/= n;
-		dy	 = dy < 0 ? -1 : 1;
-		dz	/= n;
+		n = fabs(dy); dy = dy < 0. ? -1. : 1.; dx /= n; dz /= n;
 	}
 	else
 	{
@@ -454,10 +576,9 @@ void CSG_3DView_Canvas::Draw_Line(double ax, double ay, double az, double bx, do
 	}
 
 	//-----------------------------------------------------
-	CSG_Colors	Colors(2); Colors[0] = aColor; Colors[1] = bColor;
+	CSG_Colors Colors(2); Colors[0] = aColor; Colors[1] = bColor;
 
-	//-----------------------------------------------------
-	for(double i=0.0; i<=n; i++, ax+=dx, ay+=dy, az+=dz)
+	for(double i=0.; i<=n; i++, ax+=dx, ay+=dy, az+=dz)
 	{
 		_Draw_Pixel((int)ax, (int)ay, az, Colors.Get_Interpolated(i / n));
 	}
@@ -474,33 +595,23 @@ void CSG_3DView_Canvas::Draw_Line(double ax, double ay, double az, double bx, do
 	if(	(ax < 0 && bx < 0) || (ax >= m_Image_NX && bx >= m_Image_NX)
 	||	(ay < 0 && by < 0) || (ay >= m_Image_NY && by >= m_Image_NY) )
 	{
-		return;
+		return; // completely out of area
 	}
 
-	double	dx	= bx - ax;
-	double	dy	= by - ay;
-	double	dz	= bz - az;
-
-	if( bz < 0.0 || az < 0.0 )
+	if( bz < 0. || az < 0. )
 	{
-		return;
+		return; // completely in front of screen
 	}
 
-	double	n;
+	double n, dx = bx - ax, dy = by - ay, dz = bz - az;
 
-	if( fabs(dx) > fabs(dy) && fabs(dx) > 0.0 )
+	if( fabs(dx) > fabs(dy) && fabs(dx) > 0. )
 	{
-		n	 = fabs(dx);
-		dx	 = dx < 0 ? -1 : 1;
-		dy	/= n;
-		dz	/= n;
+		n = fabs(dx); dx = dx < 0. ? -1. : 1.; dy /= n; dz /= n;
 	}
-	else if( fabs(dy) > 0.0 )
+	else if( fabs(dy) > 0. )
 	{
-		n	 = fabs(dy);
-		dx	/= n;
-		dy	 = dy < 0 ? -1 : 1;
-		dz	/= n;
+		n = fabs(dy); dy = dy < 0. ? -1. : 1.; dx /= n; dz /= n;
 	}
 	else
 	{
@@ -510,7 +621,7 @@ void CSG_3DView_Canvas::Draw_Line(double ax, double ay, double az, double bx, do
 	}
 
 	//-----------------------------------------------------
-	for(double i=0.0; i<=n; i++, ax+=dx, ay+=dy, az+=dz)
+	for(double i=0.; i<=n; i++, ax+=dx, ay+=dy, az+=dz)
 	{
 		_Draw_Pixel((int)ax, (int)ay, az, Color);
 	}
@@ -529,85 +640,89 @@ void CSG_3DView_Canvas::Draw_Line(const TSG_Point_Z &a, const TSG_Point_Z &b, in
 //---------------------------------------------------------
 void CSG_3DView_Canvas::Draw_Triangle(TSG_Triangle_Node p[3], bool bValueAsColor, double Light_Dec, double Light_Azi)
 {
-	double	A	= p[0].z * (p[1].x - p[2].x) + p[1].z * (p[2].x - p[0].x) + p[2].z * (p[0].x - p[1].x);
-	double	B	= p[0].y * (p[1].z - p[2].z) + p[1].y * (p[2].z - p[0].z) + p[2].y * (p[0].z - p[1].z);
-	double	C	= p[0].x * (p[1].y - p[2].y) + p[1].x * (p[2].y - p[0].y) + p[2].x * (p[0].y - p[1].y);
+	double s, a, C = p[0].x * (p[1].y - p[2].y) + p[1].x * (p[2].y - p[0].y) + p[2].x * (p[0].y - p[1].y);
 
-	double	s, a;
-
-	if( C != 0.0 )
+	if( C != 0. )
 	{
-		A	= - A / C;
-		B	= - B / C;
+		double A = p[0].z * (p[1].x - p[2].x) + p[1].z * (p[2].x - p[0].x) + p[2].z * (p[0].x - p[1].x);
+		double B = p[0].y * (p[1].z - p[2].z) + p[1].y * (p[2].z - p[0].z) + p[2].y * (p[0].z - p[1].z);
 
-		s	= M_PI_090 - atan(sqrt(A*A + B*B));
-		a	= A != 0.0 ? M_PI_180 + atan2(B, A) : B > 0.0 ? M_PI_270 : (B < 0.0 ? M_PI_090 : -1.0);
+		A = -A / C;
+		B = -B / C;
+
+		s = M_PI_090 - atan(sqrt(A*A + B*B));
+		a = A != 0. ? M_PI_180 + atan2(B, A) : B > 0. ? M_PI_270 : (B < 0. ? M_PI_090 : -1.);
 	}
 	else
 	{
-		s	= 0.0;
-		a	= 0.0;
+		s = 0.;
+		a = 0.;
 	}
 
 	Draw_Triangle(p, bValueAsColor, (acos(sin(s) * sin(Light_Dec) + cos(s) * cos(Light_Dec) * cos(a - Light_Azi))) / M_PI_090);
 }
 
 //---------------------------------------------------------
-#define TRIANGLE_SET_POINT(dst, index)	{	int i = index;\
-	dst[0]	= Point[i].x;\
-	dst[1]	= Point[i].y;\
-	dst[2]	= Point[i].z;\
-	switch( mode )\
+#define TRIANGLE_SET_POINT(P, i) {\
+	P[0] = Point[i].x;\
+	P[1] = Point[i].y;\
+	P[2] = Point[i].z;\
+	switch( Mode )\
 	{\
-	default:\
-		dst[3]	= Point[i].c;\
+	case 0:\
+		P[3] = Point[i].c;\
 		break;\
 	case 1:\
-		dst[3]	= Point[i].c;\
-		dst[4]	= Point[i].d;\
+		P[3] = Point[i].c;\
+		P[4] = Point[i].d;\
 		break;\
 	case 2:\
-		dst[3]	= SG_GET_R((int)Point[i].c);\
-		dst[4]	= SG_GET_G((int)Point[i].c);\
-		dst[5]	= SG_GET_B((int)Point[i].c);\
+		P[3] = SG_GET_R((int)Point[i].c);\
+		P[4] = SG_GET_G((int)Point[i].c);\
+		P[5] = SG_GET_B((int)Point[i].c);\
 		break;\
 	}\
 }
 
 //---------------------------------------------------------
-#define TRIANGLE_GET_GRADIENT(dst, A, B)	if( (dst[1] = B[1] - A[1]) > 0.0 ) {\
-	switch( mode )\
+#define TRIANGLE_GET_GRADIENT(G, A, B) if( (G[1] = B[1] - A[1]) > 0. ) {\
+	switch( Mode )\
 	{\
-	case 2:	dst[5] = (B[5] - A[5]) / dst[1];\
-	case 1:	dst[4] = (B[4] - A[4]) / dst[1];\
-	case 0:	dst[3] = (B[3] - A[3]) / dst[1];\
-			dst[2] = (B[2] - A[2]) / dst[1];\
-			dst[0] = (B[0] - A[0]) / dst[1];\
+	case 2:\
+		G[5] = (B[5] - A[5]) / G[1];\
+	case 1:\
+		G[4] = (B[4] - A[4]) / G[1];\
+	case 0:\
+		G[3] = (B[3] - A[3]) / G[1];\
+		G[2] = (B[2] - A[2]) / G[1];\
+		G[0] = (B[0] - A[0]) / G[1];\
 	}\
 }
 
 //---------------------------------------------------------
-#define TRIANGLE_SET_GRADIENT(dst, P, D)	{ double dy = y - P[1];\
-	switch( mode )\
+#define TRIANGLE_SET_GRADIENT(G, P, D) { double dy = y - P[1];\
+	switch( Mode )\
 	{\
-	case 2:	dst[5]	= P[5] + D[5] * dy;\
-	case 1:	dst[4]	= P[4] + D[4] * dy;\
-	case 0:	dst[3]	= P[3] + D[3] * dy;\
-			dst[2]	= P[2] + D[2] * dy;\
-			dst[0]	= P[0] + D[0] * dy;\
+	case 2:\
+		G[5] = P[5] + D[5] * dy;\
+	case 1:\
+		G[4] = P[4] + D[4] * dy;\
+	case 0:\
+		G[3] = P[3] + D[3] * dy;\
+		G[2] = P[2] + D[2] * dy;\
+		G[0] = P[0] + D[0] * dy;\
 	}\
 }
 
 //---------------------------------------------------------
-void CSG_3DView_Canvas::Draw_Triangle(TSG_Triangle_Node Point[3], bool bValueAsColor, double dim)
+void CSG_3DView_Canvas::Draw_Triangle(TSG_Triangle_Node Point[3], bool bValueAsColor, double Dim)
 {
-	//-----------------------------------------------------
-	if( Point[0].z < 0.0 || Point[1].z < 0.0 || Point[2].z < 0.0 )
+	if( Point[0].z < 0. || Point[1].z < 0. || Point[2].z < 0. )
 	{
 		return;	// completely in front of projection plane
 	}
 
-	CSG_Rect	r(
+	CSG_Rect r(
 		Point[0].x, Point[0].y,
 		Point[1].x, Point[1].y
 	);
@@ -616,21 +731,21 @@ void CSG_3DView_Canvas::Draw_Triangle(TSG_Triangle_Node Point[3], bool bValueAsC
 		Point[2].x, Point[2].y
 	));
 
-	if( r.Get_XMax() < 0.0 || r.Get_XMin() >= m_Image_NX
-	||	r.Get_YMax() < 0.0 || r.Get_YMin() >= m_Image_NY )
+	if( r.Get_XMax() < 0. || r.Get_XMin() >= m_Image_NX
+	||	r.Get_YMax() < 0. || r.Get_YMin() >= m_Image_NY )
 	{
 		return;	// completely off screen
 	}
 
-	if( r.Get_XRange() <= 0.0 || r.Get_YRange() <= 0.0 )
+	if( r.Get_XRange() <= 0. || r.Get_YRange() <= 0. )
 	{
 		return;	// has no area (... should draw a point ?!)
 	}
 
 	//-----------------------------------------------------
-	int	mode	= m_pDrape ? 1 : bValueAsColor ? 2 : 0;
+	int	Mode	= m_pDrape ? 1 : bValueAsColor ? 2 : 0;
 
-	double	p[3][6], d[3][6], a[6], b[6];
+	double p[3][6], d[3][6], a[6], b[6];
 
 	if( Point[0].y < Point[1].y && Point[0].y < Point[2].y )
 	{
@@ -655,109 +770,102 @@ void CSG_3DView_Canvas::Draw_Triangle(TSG_Triangle_Node Point[3], bool bValueAsC
 	TRIANGLE_GET_GRADIENT(d[1], p[0], p[1]);	// from top to midlle point
 	TRIANGLE_GET_GRADIENT(d[2], p[1], p[2]);	// from middle to bottom point
 
-	int	ay	= (int)r.Get_YMin(); if( ay < 0           ) ay	= 0; if( ay < r.Get_YMin() ) ay++;
-	int	by	= (int)r.Get_YMax(); if( by >= m_Image_NY ) by	= m_Image_NY - 1;
+	int ay = (int)(r.Get_YMin()); if( ay < 0           ) ay = 0; if( ay < r.Get_YMin() ) ay++;
+	int by = (int)(r.Get_YMax()); if( by >= m_Image_NY ) by = m_Image_NY - 1;
 
 	//-----------------------------------------------------
 	for(int y=ay; y<=by; y++)
 	{
-		if( y <= p[1][1] && d[1][1] > 0.0 )
+		if( y <= p[1][1] && d[1][1] > 0. )
 		{
 			TRIANGLE_SET_GRADIENT(a, p[0], d[0]);	//	a	= p[0] + d[0] * (y - p[0][1]);	// using CSG_Vector is, unluckily, significantly slower!!
 			TRIANGLE_SET_GRADIENT(b, p[0], d[1]);
 
 			if( a[0] < b[0] )
 			{
-				_Draw_Triangle_Line(y, a, b, dim, mode);
+				_Draw_Triangle_Line(y, a, b, Dim, Mode);
 			}
 			else
 			{
-				_Draw_Triangle_Line(y, b, a, dim, mode);
+				_Draw_Triangle_Line(y, b, a, Dim, Mode);
 			}
 		}
-		else if( d[2][1] > 0.0 )
+		else if( d[2][1] > 0. )
 		{
 			TRIANGLE_SET_GRADIENT(a, p[0], d[0]);
 			TRIANGLE_SET_GRADIENT(b, p[1], d[2]);
 
 			if( a[0] < b[0] )
 			{
-				_Draw_Triangle_Line(y, a, b, dim, mode);
+				_Draw_Triangle_Line(y, a, b, Dim, Mode);
 			}
 			else
 			{
-				_Draw_Triangle_Line(y, b, a, dim, mode);
+				_Draw_Triangle_Line(y, b, a, Dim, Mode);
 			}
 		}
 	}
 }
 
 //---------------------------------------------------------
-inline void CSG_3DView_Canvas::_Draw_Triangle_Line(int y, double a[], double b[], double dim, int mode)
+inline void CSG_3DView_Canvas::_Draw_Triangle_Line(int y, double a[], double b[], double Dim, int mode)
 {
 	if( a[0] == b[0] )
 	{
 		if( a[2] < b[2] )
 		{
-			_Draw_Pixel((int)a[0], y, a[2], _Dim_Color(Get_Color(a[3]), dim));
+			_Draw_Pixel((int)a[0], y, a[2], _Dim_Color(Get_Color(a[3]), Dim));
 		}
 		else
 		{
-			_Draw_Pixel((int)b[0], y, b[2], _Dim_Color(Get_Color(b[3]), dim));
+			_Draw_Pixel((int)b[0], y, b[2], _Dim_Color(Get_Color(b[3]), Dim));
 		}
 
 		return;
 	}
 
-	double	d[6], dx = b[0] - a[0];
+	//-----------------------------------------------------
+	double d[6], dx; int ax, bx;
 
+	ax   = (int)a[0]; if( ax <  0          ) ax = 0;
+	bx   = (int)b[0]; if( bx >= m_Image_NX ) bx = m_Image_NX - 1;
+	dx   = ax - a[0];
+	d[3] = (b[3] - a[3]) / (b[0] - a[0]);
+	d[2] = (b[2] - a[2]) / (b[0] - a[0]);
+
+	//-----------------------------------------------------
 	switch( mode )
 	{
-	case 2:	d[5]	= (b[5] - a[5]) / dx;
-	case 1:	d[4]	= (b[4] - a[4]) / dx;
-	}
-
-	d[3]	= (b[3] - a[3]) / dx;
-	d[2]	= (b[2] - a[2]) / dx;
-
-	int	ax	= (int)a[0]; if( ax <  0          )	ax	= 0;
-	int	bx	= (int)b[0]; if( bx >= m_Image_NX )	bx	= m_Image_NX - 1;
-
-	dx	= ax - a[0];
-
-	for(int x=ax; x<=bx; x++, dx++)
-	{
-		double	z	= a[2] + dx * d[2];
-
-		switch( mode )
+	default: {
+		for(int x=ax; x<=bx; x++, dx++)
 		{
-		default:
-			{
-				_Draw_Pixel(x, y, z, _Dim_Color(Get_Color(a[3] + dx * d[3]), dim));
-			}
-			break;
-
-		case 1:
-			{
-				double	Value;
-
-				if( m_pDrape->Get_Value(a[3] + dx * d[3], a[4] + dx * d[4], Value, m_Drape_Mode, false, true) )
-				{
-					_Draw_Pixel(x, y, z, _Dim_Color((int)Value, dim));
-				}
-			}
-			break;
-
-		case 2:
-			{
-				_Draw_Pixel(x, y, z, _Dim_Color(SG_GET_RGB(
-					a[3] + dx * d[3],
-					a[4] + dx * d[4],
-					a[5] + dx * d[5])
-				, dim));
-			}
-			break;
+			_Draw_Pixel(x, y, a[2] + dx * d[2], _Dim_Color(Get_Color(a[3] + dx * d[3]), Dim));
 		}
+		break; }
+
+	case 1: {
+		double Value; d[4] = (b[4] - a[4]) / (b[0] - a[0]);
+
+		for(int x=ax; x<=bx; x++, dx++)
+		{
+			if( m_pDrape->Get_Value(a[3] + dx * d[3], a[4] + dx * d[4], Value, m_Drape_Mode, false, true) )
+			{
+				_Draw_Pixel(x, y, a[2] + dx * d[2], _Dim_Color((int)Value, Dim));
+			}
+		}
+		break; }
+
+	case 2: {
+		CSG_Colors Color(2); Color.Set_Ramp(
+			SG_GET_RGB((int)a[3], (int)a[4], (int)a[5]),
+			SG_GET_RGB((int)b[3], (int)b[4], (int)b[5])
+		);
+
+		for(int x=ax; x<=bx; x++, dx++)
+		{
+			_Draw_Pixel(x, y, a[2] + dx * d[2], _Dim_Color(Color.Get_Interpolated(dx), Dim));
+		}
+		break; }
 	}
 }
 
