@@ -101,9 +101,9 @@ CWKSP_Tool_Manager::CWKSP_Tool_Manager(void)
 	);
 
 	m_Parameters.Add_Int("NODE_GENERAL",
-		"PROCESS_UPDATE", _TL("Process Update Frequency [milliseconds]"),
-		_TL(""),
-		0, 0, true
+		"PROCESS_UPDATE", _TL("Process Update Frequency"),
+		_TL("[milliseconds]"),
+		g_pSAGA->Process_Get_Frequency(), 1, true
 	);
 
 	m_Parameters.Add_Bool("NODE_GENERAL",
@@ -117,8 +117,8 @@ CWKSP_Tool_Manager::CWKSP_Tool_Manager(void)
 		"OMP_THREADS_MAX", _TL("Number of CPU Cores [# physical processors]"),
 		_TW("Number of processors to use for parallelization. Should be set to the number "
 		    "of physical processors, and not to the total number of physical and logical processors "
-			"on systems supporting hyper-threading."),
-		SG_OMP_Get_Max_Num_Threads(), 1, true, SG_OMP_Get_Max_Num_Procs(), true
+			"on systems supporting hyper-threading."
+		), SG_OMP_Get_Max_Num_Threads(), 1, true, SG_OMP_Get_Max_Num_Procs(), true
 	);
 #endif
 
@@ -169,7 +169,7 @@ CWKSP_Tool_Manager::CWKSP_Tool_Manager(void)
 	m_Parameters.Add_Int("NODE_LOOK",
 		"LOOK_TB_SIZE"	, _TL("Tool Bar Button Size"),
 		_TL("Tool bar button sizes. You need to restart SAGA to apply the changes."),
-		16, 16, true
+		24, 16, true
 	);
 
 	m_Parameters.Add_Int("NODE_LOOK",
@@ -349,11 +349,14 @@ int CWKSP_Tool_Manager::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Pa
 			if( pParameter->Cmp_Identifier("LNG_FILE_DIC")
 			||  pParameter->Cmp_Identifier("LOOK_TB_SIZE") )
 			{
-				if( DLG_Message_Confirm(_TL("Restart now ?"), _TL("Restart SAGA to apply the changes")) && g_pData->Close(true) )
+				if( DLG_Message_Confirm(_TL("Close now ?"), _TL("Restart SAGA to apply the changes")) )
 				{
 					m_Parameters.Assign_Values(pParameters);
 
-					g_pSAGA_Frame->Close(true);
+					if( g_pData->Close(true) )
+					{
+						g_pSAGA_Frame->Close();
+					}
 				}
 			}
 		}
@@ -499,11 +502,11 @@ bool CWKSP_Tool_Manager::On_Command_UI(wxUpdateUIEvent &event)
 //---------------------------------------------------------
 void CWKSP_Tool_Manager::On_Execute(wxCommandEvent &event)
 {
-	CWKSP_Tool	*pTool;
+	CWKSP_Tool *pTool = Get_Tool_byID(m_pMenu_Tools->Get_ID_Translated(event.GetId()));
 
-	if( (pTool = Get_Tool_byID(m_pMenu_Tools->Get_ID_Translated(event.GetId()))) != NULL )
+	if( pTool )
 	{
-		pTool->On_Command(ID_CMD_WKSP_ITEM_RETURN);
+		pTool->Execute(true);
 	}
 }
 
